﻿/* Emu17B2p.ahk - paste for HP17B2 under Emu42      */
/* Use: a) if not yet up start Emu42 to run HP17B2  */
/*      b) change the impact of Ctrl+v so it will   */
/*         paste numeric values from clipboard      */
/*      c) quit when Emu42 is shut down             */
/* QPL 1.0 applies, see end of file. (c) 2024 by M. */
/* "Hotel-Papa-Four-One-Charly-Xray-&-web-dot-de"   */
/* Initial version                      M. 4/15/24  */
/* Paste also negative exponents        M.  5/3/24  */
#Requires AutoHotkey v2.0
#SingleInstance Force     ; no 2nd instance, use youngest script

/* if no HP17B2 is running under Emu42 then try to start it */
if WinExist(".e17 ahk_class CEmu42")
   WinActivate
else
{
   Run("C:\Users\jg\Documents\Emu State Files\bm.e17")
   if NOT WinWaitActive(".e17 ahk_class CEmu42", , 2)
   {
      dc := MsgBox("Virtual 17B2 not up and running within 2 seconds."
             "`nAHK script ends now.", "Useless, totally", "O Iconx")
      ExitApp 9
   }
}

/* have a watchdog for Emu42 running a 17B2 */
SetTimer quitwhendue, 360

#HotIf WinActive(".e21 ahk_class CEmu42")
*LCtrl::return
*RCtrl::return
#HotIf HotIfCond.CtrlV
*c::copystack()
*v::pastevalue()

#HotIf

Class HotIfCond {
   static CtrlV {
      get {
         return getKeyState("Ctrl","P")
          && WinActive(".e17 ahk_class CEmu42")
      }
   }
}

/* * *  subroutines  * * */
quitwhendue()
{
   if not WinExist(".e17 ahk_class CEmu42")
      ExitApp 9
}

/* Since any Ctrl is captured by AHK we do need this one */
copystack()
{
   Send "^c"
}

/* paste value from clipboard */
pastevalue()
{
   Loop Parse A_Clipboard, "`n", "`r"
   {
      if StrLen(A_LoopField)
      {
         if RegExMatch(A_LoopField, "i)-?(\d[\., ']?)+(e-?\d{1,3})?", &r)
         {
            r := StrReplace(r[], "'")
            if InStr(r, ".", 1, , -1) < InStr(r, ",", 1, , -1)
            {
               r := StrReplace(r, ".")
               r := StrReplace(r, ",", ".")
            }
            else r := StrReplace(r, ",")
            r := StrReplace(r, " ", "")
            if Ord(r) = 45
               r := SubStr(r, 2, 1) "c" SubStr(r, 3)
            Send StrReplace(r, "-", "{NumpadSub}")
         }
         else SoundBeep
      }
      else SoundBeep
      break          ; use 1st record only
   }
}

/* * * * *  THE Q PUBLIC LICENSE version 1.0  * * * * *
Copyright (C) 1999 Trolltech AS, Norway.

Everyone is permitted to copy and distribute this license document. The intent of this license is to establish freedom to share and change the software regulated by this license under the open source model.

This license applies to any software containing a notice placed by the copyright holder saying that it may be distributed under the terms of the Q Public License version 1.0. Such software is herein referred to as the Software. This license covers modification and distribution of the Software, use of third-party application programs based on the Software, and development of free software which uses the Software.

Granted Rights

  1.
    You are granted the non-exclusive rights set forth in this license provided you agree to and comply with any and all conditions in this license. Whole or partial distribution of the Software, or software items that link with the Software, in any form signifies acceptance of this license.
  2.
    You may copy and distribute the Software in unmodified form provided that the entire package, including - but not restricted to - copyright, trademark notices and disclaimers, as released by the initial developer of the Software, is distributed.
  3.
    You may make modifications to the Software and distribute your modifications, in a form that is separate from the Software, such as patches. The following restrictions apply to modifications:
    a. Modifications must not alter or remove any copyright notices in the Software.
    b. When modifications to the Software are released under this license, a non-exclusive royalty-free right is granted to the initial developer of the Software to distribute your modification in future versions of the Software provided such versions remain available under these terms in addition to any other license(s) of the initial developer.
  4.
    You may distribute machine-executable forms of the Software or machine-executable forms of modified versions of the Software, provided that you meet these restrictions:
    a. You must include this license document in the distribution.
    b. You must ensure that all recipients of the machine-executable forms are also able to receive the complete machine-readable source code to the distributed Software, including all modifications, without any charge beyond the costs of data transfer, and place prominent notices in the distribution explaining this.
    c. You must ensure that all modifications included in the machine-executable forms are available under the terms of this license.
  5.
    You may use the original or modified versions of the Software to compile, link and run application programs legally developed by you or by others.
  6.
    You may develop application programs, reusable components and other software items that link with the original or modified versions of the Software. These items, when distributed, are subject to the following requirements:
    a. You must ensure that all recipients of machine-executable forms of these items are also able to receive and use the complete machine-readable source code to the items without any charge beyond the costs of data transfer.
    b. You must explicitly license all recipients of your items to use and re-distribute original and modified versions of the items in both machine-executable and source code forms. The recipients must be able to do so without any charges whatsoever, and they must be able to re-distribute to anyone they choose.
    c. If the items are not available to the general public, and the initial developer of the Software requests a copy of the items, then you must supply one.

Limitations of Liability
In no event shall the initial developers or copyright holders be liable for any damages whatsoever, including - but not restricted to - lost revenue or profits or other direct, indirect, special, incidental or consequential damages, even if they have been advised of the possibility of such damages, except to the extent invariable law, if any, provides otherwise.

No Warranty
The Software and this license document are provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

Choice of Law
This license is governed by the Laws of any member state of the European Union.
* * * * *  End Of File  * * * * */